% Main script
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%      Exact numerical integration of the master     %%%%%%%%%%%%
%%%%%%%%%%%        equation in some models of stochastic       %%%%%%%%%%%%
%%%%%%%%%%%                     epidemiology                   %%%%%%%%%%%%
%%%%%%%%%%%              FIGURE GENERATION SOFTWARE            %%%%%%%%%%%%
%%%%%%%%%%%             Authors: Jenkinson/Goutsias            %%%%%%%%%%%%
%%%%%%%%%%%              Last Modified: 01/13/2012             %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;
close all;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize model parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

makeFigs  = 1;       % 1 makes figs, 0 does not make figs 
saveMovie = 1;       % 0 does not save movie
                     % 1 saves movie as GIF (requires image processing toolbox)
findSteadyState = 0; % 0 normal simulation
                     % 1 finds steady-state distribution and nothing else 

s0 = 762;            % initial number of susceptible individuals
i0 = 1;              % initial number of infected individuals
r0 = 0;              % initial number of resistant individuals 

M = 2;               % number of reactions

k1 = 0.00218;        % 1/day, S + I -> 2I
k2 = 0.44036;        % 1/day,     I -> R

% define time steps
tf = 25;             % days
if findSteadyState
    % alter parameters
    makeFigs  = 0;
    saveMovie = 0;
    tf        = 50;
end
tau = 0.01;        % days
steps = tf/tau;    % number of steps

% determine sample space Z
hyperCubeOrigin = zeros(M,1); % origin of hypercube
hyperCubeExtent = [762;763];  % extent of hypercube
K               = 763*764;    % number of states

% define initial probability 
pPrev    = zeros(K,1);
pPrev(1) = 1;

% make figure object and handles
if makeFigs
    fig = figure('Position',[50 50 650 700]);
    set(fig,'nextplot','replacechildren','visible','off')
    h1 = subplot(2,2,[1 2]);
    h2 = subplot(2,2,3);
    h3 = subplot(2,2,4);
    set(h1,'Position',[.1 .45 .85 .5]);
    set(h2,'Position',[.1 .1  .35 .25]);
    set(h3,'Position',[.6 .1  .35 .25]);
    
    if saveMovie
        surfc(h1,rand(100,100))
        set(h1,'XGrid','on');
        set(h1,'YGrid','on');
        set(h1,'ZGrid','off');
        colormap(h1,'jet')
        shading(h1,'interp')
        f = getframe(fig);
        [im,map] = rgb2ind(f.cdata,256);
        im(1,1,1,floor(steps/20)) = 0;
    end
    
    %initialize
    count         = 1;
    mean          = zeros(floor(steps/20),2);
    meanI         = zeros(floor(steps/20),1);
    standDevI     = zeros(floor(steps/20),1);    
    meanS         = zeros(floor(steps/20),1);
    standDevS     = zeros(floor(steps/20),1);
    meanR         = zeros(floor(steps/20),1);
    standDevR     = zeros(floor(steps/20),1);
    meanRcond     = zeros(floor(steps/20),1);
    standDevRcond = zeros(floor(steps/20),1);
    timePoints    = zeros(floor(steps/20),1);
    probIeq0      = zeros(floor(steps/20),1);
    
    % real data points
    dataPoints = round(50*[0,1/8,1/2,1+15/32,4+3/8,5.75,5+1/32,4+5/8,...
                       3+25/32,2+15/32,1+3/8,17/32,7/32,1/16]);
    dataTime   = 1:14; 
end

if findSteadyState==1
    count = 1;
    probIeq0 = zeros(floor(steps/20),1);
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve system
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

% find implicit Euler matrix
IminusDeltaA = genMatrix(k1,k2,s0,i0,K,hyperCubeOrigin,hyperCubeExtent,M,tau);

% perform implicit Euler solver steps

for index = 1:steps
    %
    % Do Euler solver steps
    %
    pNext = IminusDeltaA\pPrev; % matlab automatically performs recursive substitution
    pPrev = pNext;              % prepare for next time step
    
    %
    % Generate figures
    %
    
    if mod(index,20)==0 && findSteadyState==1
         [Rprobs,SIprobs]  = mapProbs(pNext,hyperCubeOrigin,hyperCubeExtent,s0,i0);
         probIeq0(count)   = sum(SIprobs(1:end,1));
         timePoints(count) = 0.2*count;
         count             = count+1;
    end
        
    if mod(index,20)==0 && makeFigs
        %
        % find probabilities in Z and X sample spaces
        %
        [Rprobs,SIprobs] = mapProbs(pNext,hyperCubeOrigin,hyperCubeExtent,s0,i0);
        
        %
        % Compute conditional probabilities and means
        %
        probIeq0(count) = sum(SIprobs(1:end,1));
        mean(count,1)   = (0:762)*sum(SIprobs(1:end,2:end),2)/(1-probIeq0(count));
        mean(count,2)   = sum(SIprobs(1:end,2:end),1)*(1:763)'/(1-probIeq0(count));
        
        %
        % Compute marginal mean and standard deviation of infected pupils
        %
        meanI(count)     = sum(SIprobs,1)*(0:763)';
        standDevI(count) = sqrt(sum(SIprobs,1)*(((0:763)-meanI(count)).^2)');
        
        %
        % Compute marginal mean and standard deviation of resistant pupils
        %
        meanR(count)     = (0:763)*Rprobs;
        standDevR(count) = sqrt((((0:763)-meanR(count)).^2)*Rprobs);
        
        %
        % Compute marginal mean and standard deviation of susceptible pupils
        %
        meanS(count)     = (0:762)*sum(SIprobs,2);
        standDevS(count) = sqrt((((0:762)-meanS(count)).^2)*sum(SIprobs,2));
        
        %
        % Compute conditional R probability
        %
        condRprobs = zeros(764,1);
        for Sindex = 1:size(SIprobs,1)
            for Iindex = 2:size(SIprobs,2)
                if SIprobs(Sindex,Iindex)>0
                    rValue = 763 - (Sindex-1) - (Iindex-1); % R = Q-S-I
                    condRprobs(rValue+1) = condRprobs(rValue+1)+...
                        SIprobs(Sindex,Iindex)/(1-probIeq0(count));
                end
            end
        end
        % find mean and std of R
        meanRcond(count)     = (0:763)*condRprobs;
        standDevRcond(count) = sqrt((((0:763)-meanRcond(count)).^2)*condRprobs);
        
        %
        % make plots
        %
        
        surfc(h1,SIprobs(1:end,2:end)/(1-probIeq0(count)))
        set(h1,'XGrid','on');
        set(h1,'YGrid','on');
        set(h1,'ZGrid','off');
        colormap(h1,'jet')
        shading(h1,'interp')
        xlabel(h1,'infected pupils (I)');
        ylabel(h1,'susceptible pupils (S)');
        zlabel(h1,'probability');
        title(h1,'Pr[ S, I | I > 0 ]');
        axis(h1,'tight')
        
        hold(h2,'off');
        plot(h2,0:763,condRprobs,'LineWidth',2);
        hold(h2,'on');
        plot(h2,0,1.1*max(condRprobs),'w.')
        xlabel(h2,' recovered pupils (R)');
        ylabel(h2,'probability');
        title(h2,'Pr[ R | I > 0 ]')
        axis(h2,'tight')
        
        hold(h3,'off');
        plot(h3,mean(1:count-1,1),mean(1:count-1,2),'b--','LineWidth',2)
        hold(h3,'on');
        plot(h3,mean(count,1),mean(count,2),'ro','LineWidth',4)
        set(fig,'CurrentAxes',h3)
        text(355,550,['Time = ' num2str(count*.2) ' days'],...
                      'HorizontalAlignment','center')
        text(355,450,['Pr[ I = 0 ] = ' num2str(probIeq0(count))],...
                      'HorizontalAlignment','center')
        xlabel(h3,'E[ S | I > 0 ]');
        ylabel(h3,'E[ I | I > 0 ]');
        axis(h3,[0 762 0 763])
        
        timePoints(count) = 0.2*count;

        %
        % Save frames for animation of solution (requires Image Processing 
        % toolbox)
        %
        
        f = getframe(fig);
        if saveMovie
            im(:,:,1,count) = rgb2ind(f.cdata,map);
        end
        count = count+1;
    end
end

%
% save movie and make figures
%

if makeFigs
    %estimate using the Gillespie algorithm
    [meanInfected,stdInfected,meanResis,stdResis,meanSusc,...
        stdSusc,Gilltime] = gillespie(tf);
    makePlots
end

% Steady state calculations

if findSteadyState
    %
    % find probabilities in Z and SI planes
    %
    [Rprobs,SIprobs] = mapProbs(pNext,hyperCubeOrigin,hyperCubeExtent,s0,i0);

    %
    % plot probability of no infection
    %
    hPI0 = figure;
    plot([0,timePoints],[0;probIeq0],'k','Linewidth',2);
    xlabel('time (days)');
    ylabel('probability of extinction')
    axis([0 50 0 1.1])
    saveas(hPI0,'results/probExtinct.fig')

    %
    % plot joint probability of S and I=0 at steady state
    %
    hSS = figure;
    bar(0:762,SIprobs(1:end,1),'LineWidth',2);
    xlabel('pupils that escape infection');
    ylabel('probability')
    axis([0 800 0 0.25])
    saveas(hSS, 'results/steadyState.fig');

    %
    % Compute numerical details of steady state distribution
    %
    meanEscapeInfection    = (0:762)*SIprobs(1:end,1);
    percentEscape40orLess  = sum(SIprobs(1:40,1))
    percentEscape753orMore = sum(SIprobs(753:763,1))
end