% Solve SIR model using RIE method with variable step-size
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%      Exact numerical integration of the master     %%%%%%%%%%%%
%%%%%%%%%%%       equation in some models of stochastic        %%%%%%%%%%%%
%%%%%%%%%%%                    epidemiology                    %%%%%%%%%%%%
%%%%%%%%%%%            RIE METHOD (VARIABLE STEP-SIZE)         %%%%%%%%%%%%
%%%%%%%%%%%              Authors: Jenkinson/Goutsias           %%%%%%%%%%%%
%%%%%%%%%%%               Last Modified: 01/13/2012            %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;
close all;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize simulation parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

tic;        % start timer

tol = 1e-3; % TOL parameter for variable timestepper 
tf  = 25;   % simulation time in days

display('Running RIE Method (Variable Step-Size). Please wait...');

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize model parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

s0 = 762;            % initial number of susceptible individuals
i0 = 1;              % initial number of infected individuals 
r0 = 0;              % initial number of recovered individuals 

M = 2;               % number of reactions

k1 = 0.00218;        % 1/day, S + I -> 2I
k2 = 0.44036;        % 1/day,     I -> R

% determine sample space Z
hyperCubeOrigin = zeros(M,1); % origin of hypercube
hyperCubeExtent = [762;763];  % extent of hypercube
K               = 763*764;    % number of states

% define initial probability 
pPrev    = zeros(K,1);
pPrev(1) = 1;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve system
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

% find implicit Euler matrix
A = genMatrix(k1,k2,s0,i0,K,hyperCubeOrigin,hyperCubeExtent,M);

I=speye(size(A));
% perform implicit Euler solver steps

tnow=0;
tot_error = zeros(K,1);
tau=tol;

while tnow < tf
    %
    % Do Euler solver steps
    %
    pNext1 = (I-tau*A)\pPrev; % matlab automatically performs recursive substitution
    
    pTemp = (I-tau*A/2)\pPrev;
    
    pNext2 = (I-tau*A/2)\pTemp;
    
    
    error_loc = (pNext2-pNext1);
    
    normError = norm(error_loc,1);
    
    tol_loc = tol; 
    
    if normError <= tol_loc %accept step, increase step size
        
        tot_error = tot_error + error_loc;
        
        %
        % set new probability via Richardson Extrapolation
        %
        pPrev = (2*pNext2)-pNext1; 
        if min(pPrev)<0 % ensure that Richardson extrapolation produces a probability vector
            pPrev = pNext2;
        end
        
        %
        % update time
        %
        tnow  = tnow+tau; 

        %
        % adaptively choose stepsize
        %
        factor = min(10,.9*sqrt(tol_loc/normError)); 
        tau = tau*factor;
        
        %
        % round stepsize for numerical stability
        %
        s = 10^(floor(log10(tau))-1);
        tau = ceil(tau/s) * s;

    else % do not accept step, decrease step size
        
        factor = max(1/5,.9*sqrt(tol_loc/normError));
        tau = tau*factor;% set stepsize
        
        %
        % round stepsize for numerical stability
        %
        s = 10^(floor(log10(tau))-1);
        tau = ceil(tau/s) * s;
        
    end
end

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Map probability vector to SI plane
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

SIprobs = mapProbs(pNext2,hyperCubeOrigin,hyperCubeExtent,s0,i0);

final_error_est = norm(tot_error,1); % compute the final error estimate

timeElapsed = toc; % store the time spent in comptutaion

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute L2 error
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

load ../SIprobsGroundTruth.mat

FinalError = sqrt(sum(sum((SIprobsTruth-SIprobs).^2)));

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Save results to file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

saveResultsToFile;